/*!
	@file
	@author		Albert Semenov
	@date		10/2008
*/

#ifndef MYGUI_LAST_HEADER_H_
#define MYGUI_LAST_HEADER_H_

#include "MyGUI_Prerequest.h"

// used to enable deprecated warnings, disabled by Ogre
#if MYGUI_COMPILER == MYGUI_COMPILER_MSVC

#	pragma warning( default: 4996)

#endif

#endif // MYGUI_LAST_HEADER_H_
